
import java.awt.*;
import java.awt.event.*;
import java.applet.*;


import calc.Complex;
import calc.DoCalc;
import show.ComplexDisplay;


public class CApplet extends Applet {

  public final String HELP_TEXT =
  "f(z): \n"
 +"Any function containing Re z, Im z, z^n, sin z, cos z, exp z, zeta z\n"
 +"can be evaluated.\n"
 +"Examples: 3*z^3 - 4*z^2 + 1.5,  Re(exp(1/z)),  1/(zeta z)\n\n"
 +"Display as:\n"
 +"Choose any sort of diagram. The exact function value can be\n"
 +"determined in 2D-displays by moving the mouse on to the z\n"
 +"coords in question.\n\n"
 +"The different types of Display are:\n\n"
 +"Altitude Diagram: Shows |f(z)| as grey-level in the gaussian plane\n"
 +"(black: |f(z)| low, white: |f(z)| high)\n\n"
 +"Vectorfield: Shows f(z) as vector (with unified length) in the\n"
 +"gaussian plane\n\n"
 +"2D-Grid: Shows f(z) as transformed rectangular grid (for\n"
 +"example, exp(z) produces circles)\n\n"
 +"3D-Grid: Shows |f(z)| as 'height' above the gaussian plane\n\n"
 +"Range:\n"
 +"The real and imaginary range of z, for which the function values\n"
 +"are calculated\n\n"
 +"Steps:\n"
 +"The number of vertices in each dimension. For example,\n"
 +"Steps=50 calculates f(z) for a grid of 50x50 values.\n\n"
 +"Any problems, bugs, ideas? Write email to tim@paehler.org\n";

  /** used for storing the Range in real direction */
  protected double[] realRange = {-10.0d, 10.0d};

  /** used for storing the Range in imaginary direction */
  protected double[] imagRange = {-10.0d, 10.0d};

  /** stores the values for caching */
  protected Complex[][] values;

  /** stores the chosen grid size (also for caching) */
  protected int currentSteps = 50;

  /** holds the constructed operational tree for the function */
  protected DoCalc result;

  /** stores the function string (also for caching) */
  String currentFunction = "";

  static class Debug {
    private static void out(String str) {
      System.out.println(str);
    }
  }

  /** an inner class, which is forwarded as front end to
      DoCalc.calculateFor */
  public class CalculatorFrontend implements calc.Calculator {
    DoCalc function;
    public CalculatorFrontend(DoCalc function){
      this.function = function;
    }
    public Complex calculateForCurrentFunction(Complex c){
      return function.calculateFor(c);
    }
  }


  //UI-Components
  Label functionLabel = new Label("   f(z) := ");
  TextField functionField = new TextField("exp(1/z)",30);
  Label as = new Label(" display as: ");
  Choice displayChoice = new Choice();
  Button helpButton = new Button("Help");
  Label rangeLabel1 = new Label("real:   from");
  TextField realRangeMin = new TextField("-10.0",5);
  Label to1 = new Label(" to");
  TextField realRangeMax = new TextField("10.0",5);
  Label rangeLabel2 = new Label(";   imaginary:   from");
  TextField imagRangeMin = new TextField("-10.0",5);
  Label to2 = new Label(" to");
  TextField imagRangeMax = new TextField("10.0",5);
  Label gridSizeLabel = new Label(" Steps:");
  TextField gridSizeField = new TextField("100",5);
  Panel prompt1 = new Panel();
  Panel prompt2 = new Panel();
  Panel prompt = new Panel();
  ComplexDisplay display = new show.AltitudeDisplay();

        
  public void init(){
	
    //UI Initialization
    setLayout(new BorderLayout());
    prompt.setLayout(new GridLayout(2,1));
    prompt1.add(helpButton);
    prompt1.add(functionLabel);
    prompt1.add(functionField);
    prompt1.add(as);
    displayChoice.add("Altitude Diagram");
    displayChoice.add("Vectorfield");
    displayChoice.add("2D-Grid");
    displayChoice.add("3D-Grid");

    prompt1.add(displayChoice);
    prompt2.add(rangeLabel1);
    prompt2.add(realRangeMin);
    prompt2.add(to1);
    prompt2.add(realRangeMax);
    prompt2.add(rangeLabel2);
    prompt2.add(imagRangeMin);
    prompt2.add(to2);
    prompt2.add(imagRangeMax);
    prompt2.add(gridSizeLabel);
    prompt2.add(gridSizeField);

	
    prompt.add(prompt1);
    prompt.add(prompt2);
    add(prompt, BorderLayout.SOUTH);
    //add(display, BorderLayout.CENTER);

    //Controller Initialization
    ActionListener ctrl =  new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	calculate(functionField.getText());
      }};
    
    // every action should lead to recalculation
    functionField.addActionListener(ctrl);
    gridSizeField.addActionListener(ctrl);
    realRangeMin.addActionListener(ctrl);
    realRangeMax.addActionListener(ctrl);
    imagRangeMin.addActionListener(ctrl);
    imagRangeMax.addActionListener(ctrl);
    displayChoice.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e){;
      calculate(functionField.getText());}});
    helpButton.addActionListener(new ActionListener(){
	public void actionPerformed(ActionEvent e){
	  displayHelp();
	}});
  }

  /** for debugging purposes */
  public void processEvent(AWTEvent e){
    System.out.println(e);
    super.processEvent(e);
  }

  /** show helpText */
  public void displayHelp(){
    Frame helpFrame = new Frame("Help");
    TextArea helpArea = new TextArea(HELP_TEXT);
    helpFrame.add(helpArea);
    helpFrame.setSize(440,320);
    helpFrame.setVisible(true);
  }

  void calculate(String expr){
    System.out.println("Calculating: "+expr+"!");
    long time = System.currentTimeMillis();

    // see, if anything since last calculation has changed (i.e. has the
    // user just changed the display?)

    if(!(currentFunction.equals(expr) &&
	 realRange[0] == Double.parseDouble(realRangeMin.getText()) &&
	 realRange[1] == Double.parseDouble(realRangeMax.getText()) &&
	 imagRange[0] == Double.parseDouble(imagRangeMin.getText()) &&
	 imagRange[1] == Double.parseDouble(imagRangeMax.getText()) &&
	 currentSteps == Integer.parseInt(gridSizeField.getText()))){

      // initiate new calculation
      currentFunction = expr;
      result = new DoCalc(expr);

      // parse Ranges
      try {
	currentSteps = Integer.parseInt(gridSizeField.getText());
	result.setSteps(currentSteps);
	realRange[0] = Double.parseDouble(realRangeMin.getText());
	realRange[1] = Double.parseDouble(realRangeMax.getText());
	imagRange[0] = Double.parseDouble(imagRangeMin.getText());
	imagRange[1] = Double.parseDouble(imagRangeMax.getText());
      } catch(NumberFormatException e){e.printStackTrace();}

      result.setRange(realRange,imagRange);
      result.calculate();
      values = result.getValues();
      
      time = System.currentTimeMillis() - time;
      System.out.println ("Done in "+time/1000 +" seconds.");
    }

    // renew and update the display
    Frame displayFrame = new Frame("f(z) = "+currentFunction+"   Range: ["
				   +realRange[0]+","+realRange[1]+"] x ["
				   +imagRange[0]+"i,"+imagRange[1]+"i] ");
    displayFrame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e){
	((Frame)e.getSource()).setVisible(false);
      }
    });
    System.out.println(displayChoice.getSelectedItem());
    if(displayChoice.getSelectedItem().equals("Vectorfield"))
      display = new show.VectorFieldDisplay();
    else if(displayChoice.getSelectedItem().equals("Altitude Diagram"))
      display = new show.AltitudeDisplay();
    else if(displayChoice.getSelectedItem().equals("2D-Grid"))
      display = new show.GridDisplay();
    else if(displayChoice.getSelectedItem().equals("3D-Grid"))
      display = new show.Canvas3DDisplay();
    display.setCalculator(new CalculatorFrontend(result));
    display.setRange(realRange,imagRange);
    display.setValues(values);
    Debug.out("Done.");
    displayFrame.add(display);
    displayFrame.setSize(800,600);
    displayFrame.setVisible(true);
	
  }
    

  public static void main(String[] args){
    CApplet ds = new CApplet();
    Frame frame = new Frame("CApplet");
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e){
	System.exit(0);
      }
    });
    frame.add(ds,BorderLayout.CENTER);
    frame.setSize(900,100);
    ds.init();
    ds.start();
    ds.validate();
    frame.setVisible(true);
  }
}



    
