package calc.zeta;

import java.util.*;
import calc.*;

/* This class builds the appropriate Bernoulli-Polynom for a given degree 
   using the calc-package */

public class BernoulliPolynomBuilder {

  private static double[] B1Coeffs = {-0.5d, 1.0d};
  protected static final Polynom B1 = new Polynom(B1Coeffs);
  public static double[] BernoulliNumber;
  public static Polynom[] BernoulliPolynomials;
  public static double[] Facs;

  public static double fac(int i){
    // use cache
    if(i <= 0)
      return 1;
    if(Facs == null || Facs.length <= i)
      Facs = new double[i+1];
    if(Facs[i] != 0)
      return Facs[i];
    Facs[i] = i * fac(i-1);
    //System.out.println("fac("+i+") = "+Facs[i]);
    return Facs[i];
  }
  
  public static double binomial(int n, int k){
    if( k > n)
      return 0;
    return fac(n)/(fac(k)*fac(n-k));
  }


  public static double getBernoulliNumber(int q){
    if(q <= 0)
      return -1;
    //use cache
    if(BernoulliNumber != null && BernoulliNumber.length  > q){
      //System.out.println("# for "+q+"is : "+BernoulliNumber[q]);
      return BernoulliNumber[q]; 
    }
    BernoulliNumber = new double[q+1];    

    BernoulliNumber[1] = -0.5;
    for(int i = 2; i <= q; i++){

      BernoulliNumber[i] = -1;
      for(int j = 1; j < i; j++){
	//System.out.println("i ="+i+", j ="+j+",\t binomial("+(i+1)+","+j
	// +") = "+ binomial(i+1,j));	
	BernoulliNumber[i] -= binomial(i+1,j) * BernoulliNumber[j];
	//System.out.println("BernoulliNumber["+i+"] = "+BernoulliNumber[i]);
      }
      BernoulliNumber[i] /= binomial(i+1,i);
      //System.out.println("B_"+i+" = "+BernoulliNumber[i]);
    }

    // it is more numerically stable to use the (mis)calculated (non-zero) 
    // values for 2n+1 Bernoulli-Numbers, but if asked specifically, we 
    // return the (theoretically proven) zero values

    double retVal = 0; 
    if(q == 1 || q % 2 == 0)
      retVal = BernoulliNumber[q];

    //System.out.println("# for "+q+" is "+retVal);
    return retVal;
  }
    

  public static Polynom buildPolynom(int q) {

    //use cache
    if(BernoulliPolynomials != null && BernoulliPolynomials.length  > q)
    return new Polynom(BernoulliPolynomials[q]); 

    BernoulliPolynomials = new Polynom[q+2];    
    Polynom BernoulliPolynom = new Polynom(B1);
    BernoulliPolynomials[1] =  BernoulliPolynom;

    if(q > 1){
      for(int r = 1; r < q; r++){
	BernoulliPolynom.multiply(r+1);
	BernoulliPolynom.integrate(getBernoulliNumber(r+1));
	BernoulliPolynomials[r+1] = new Polynom(BernoulliPolynom);
      }
    }
    // do not return the original reference 
    return new Polynom(BernoulliPolynom);
  }


  /** for testing purposes */
  public static void main(String[] args){
    System.out.println(""+buildPolynom(Integer.parseInt(args[0])));
    //System.out.println(getBernoulliNumber(Integer.parseInt(args[0])));
  }
}
