package show;

import java.awt.*;
import java.awt.event.*;

import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.ColorCube;
import calc.DoCalc;
import calc.Complex;


public class Canvas3DDisplay extends ComplexDisplay {
	
	
  private TransformGroup viewTG = null;
  TransformGroup contentTG = null;
  private Complex[][] values;
  private Canvas3D plot3d = null;
  private DirPanel dirPanel = new DirPanel();
	
  /** a panel containing the arrow-buttons and doing the movement */
  private class DirPanel extends Panel implements KeyListener {
		Label xcoord = new Label("x:                         ");
		Label ycoord = new Label("y:                         ");
		Label zcoord = new Label("z:                         ");
		
		/** used as constants: when left or right arrow is pushed, view is rotated
		 by 3 degrees */
		private final Transform3D ROTLEFT;
		private final Transform3D ROTRIGHT;
		private final Transform3D ROTUP;
		private final Transform3D ROTDOWN;
		
		/** Constructor: initialize GUI, add Actionlisteners for Buttons */
		DirPanel(){
			
			// initialize constants
			Matrix4d m = new Matrix4d();
			m.setIdentity();
			m.rotY(Math.PI/120.0);
			ROTLEFT = new Transform3D(m);
			m.setIdentity();
			m.rotY(-Math.PI/120.0);
			ROTRIGHT = new Transform3D(m);
			m.setIdentity();
			m.rotX(Math.PI/120.0);
			ROTUP = new Transform3D(m);
			m.setIdentity();
			m.rotX(-Math.PI/120.0);
			ROTDOWN = new Transform3D(m);
			
			Panel coords = new Panel();
			Panel grid1 = new Panel();
			Panel grid2 = new Panel();
			Panel grid3 = new Panel();
			Panel grid4 = new Panel();
			Panel grid5 = new Panel();
			coords.setLayout(new GridLayout(4,1));
			coords.add(new Label("Position"));
			coords.add(xcoord);
			coords.add(ycoord);
			coords.add(zcoord);
			grid1.setLayout(new BorderLayout());
			grid2.setLayout(new BorderLayout());
			grid3.setLayout(new BorderLayout());
			grid4.setLayout(new BorderLayout());
			grid5.setLayout(new BorderLayout());
			Button up = new Button("up");
			Button down = new Button("down");
			Button forward = new Button("forward");
			Button backward = new Button("backward");
			Button left = new Button("left");
			Button right = new Button("right");
			Button rollUp = new Button("roll up");
			Button rollDown = new Button("roll down");
			Button rollLeft = new Button("turn left");
			Button rollRight = new Button("turn right");
			Button flatten = new Button("flatten");
			Button raise = new Button("raise");
			Button scaleUp = new Button("scale up");
			Button scaleDown = new Button("scale down");
			
			up.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							up();
						}
					});
			down.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							down();
						}
					});
			forward.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							forward();
						}
					});
			backward.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							backward();
						}
					});
			left.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							left();
						}
					});
			right.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							right();
						}
					});
			rollUp.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							rollUp();
						}
					});
			rollDown.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							rollDown();
						}
					});
			rollLeft.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							rollLeft();
						}
					});
			rollRight.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							rollRight();
						}
					});
			flatten.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							flatten();
						}
					});
			raise.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							raise();
						}
					});
			scaleUp.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							scaleUp();
						}
					});
			scaleDown.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e){
							scaleDown();
						}
					});
			
			grid1.add(up,BorderLayout.NORTH);
			grid1.add(down,BorderLayout.SOUTH);
			grid2.add(forward,BorderLayout.NORTH);
			grid2.add(backward,BorderLayout.SOUTH);
			grid2.add(left,BorderLayout.WEST);
			grid2.add(right,BorderLayout.EAST);
			grid3.add(rollUp,BorderLayout.NORTH);
			grid3.add(rollDown,BorderLayout.SOUTH);
			grid3.add(rollLeft,BorderLayout.WEST);
			grid3.add(rollRight,BorderLayout.EAST);
			grid4.add(raise,BorderLayout.NORTH);
			grid4.add(flatten,BorderLayout.SOUTH);
			grid5.add(scaleUp,BorderLayout.NORTH);
			grid5.add(scaleDown,BorderLayout.SOUTH);
			add(coords);
			add(grid1);
			add(grid2);
			add(grid3);
			add(grid4);
			add(grid5);
		}
		
		void setCoords(Vector3f p){
			xcoord.setText("x: "+p.x);
			ycoord.setText("y: "+p.y);
			zcoord.setText("z: "+p.z);
		}
		// the movement methods:
		
		public void up(){
			Transform3D vt = new Transform3D();
			viewTG.getTransform(vt);
			Transform3D tr = new Transform3D();
			tr.setTranslation(new Vector3f(0.0f,0.05f,0.0f));
			vt.mul(tr);
			viewTG.setTransform(vt);
			Vector3f newCoords = new Vector3f();;
			vt.get(newCoords);
			setCoords(newCoords);
		}
		
		public void down(){
			Transform3D vt = new Transform3D();
			viewTG.getTransform(vt);
			Transform3D tr = new Transform3D();
			tr.setTranslation(new Vector3f(0.0f,-0.05f,0.0f));
			vt.mul(tr);
			viewTG.setTransform(vt);
			Vector3f newCoords = new Vector3f();;
			vt.get(newCoords);
			setCoords(newCoords);
		}
		
		public void forward(){
			Transform3D vt = new Transform3D();
			viewTG.getTransform(vt);
			Transform3D tr = new Transform3D();
			tr.setTranslation(new Vector3f(0.0f,0.0f,-0.05f));
			vt.mul(tr);
			viewTG.setTransform(vt);
			Vector3f newCoords = new Vector3f();;
			vt.get(newCoords);
			setCoords(newCoords);
		}
		
		public void backward(){
			Transform3D vt = new Transform3D();
			viewTG.getTransform(vt);
			Transform3D tr = new Transform3D();
			tr.setTranslation(new Vector3f(0.0f,0.0f,0.05f));
			vt.mul(tr);
			viewTG.setTransform(vt);
			Vector3f newCoords = new Vector3f();;
			vt.get(newCoords);
			setCoords(newCoords);
		}
		
		public void left(){
			Transform3D vt = new Transform3D();
			viewTG.getTransform(vt);
			Transform3D tr = new Transform3D();
			tr.setTranslation(new Vector3f(-0.05f,0.0f,0.0f));
			vt.mul(tr);
			viewTG.setTransform(vt);
			Vector3f newCoords = new Vector3f();;
			vt.get(newCoords);
			setCoords(newCoords);
		}
		
		public void right(){
			Transform3D vt = new Transform3D();
			viewTG.getTransform(vt);
			Transform3D tr = new Transform3D();
			tr.setTranslation(new Vector3f(0.05f,0.0f,0.0f));
			vt.mul(tr);
			viewTG.setTransform(vt);
			Vector3f newCoords = new Vector3f();;
			vt.get(newCoords);
			setCoords(newCoords);
		}
		
		public void rollUp(){
			Transform3D vt = new Transform3D();
			viewTG.getTransform(vt);
			vt.mul(ROTUP);
			viewTG.setTransform(vt);
			Vector3f newCoords = new Vector3f();;
			vt.get(newCoords);
			setCoords(newCoords);
		}
		
		public void rollDown(){
			Transform3D vt = new Transform3D();
			viewTG.getTransform(vt);
			vt.mul(ROTDOWN);
			viewTG.setTransform(vt);
			Vector3f newCoords = new Vector3f();;
			vt.get(newCoords);
			setCoords(newCoords);
		}
		
		public void rollLeft(){
			Transform3D vt = new Transform3D();
			viewTG.getTransform(vt);
			vt.mul(ROTLEFT);
			viewTG.setTransform(vt);
			Vector3f newCoords = new Vector3f();;
			vt.get(newCoords);
			setCoords(newCoords);
		}
		
		public void rollRight(){
			Transform3D vt = new Transform3D();
			viewTG.getTransform(vt);
			vt.mul(ROTRIGHT);
			viewTG.setTransform(vt);
			Vector3f newCoords = new Vector3f();;
			vt.get(newCoords);
			setCoords(newCoords);
		}
		
		/** scale the y-Value of the content by 1.25 */
		public void raise(){
			Transform3D vt = new Transform3D();
			contentTG.getTransform(vt);
			Matrix4d m = new Matrix4d();
			vt.get(m);
			m.m11*=1.25;
			vt.set(m);
			contentTG.setTransform(vt);
		}
		
		/** scale the y-Value of the content by 0.8 */
		public void flatten(){
			Transform3D vt = new Transform3D();
			contentTG.getTransform(vt);
			Matrix4d m = new Matrix4d();
			vt.get(m);
			m.m11 *= 0.8;
			vt.set(m);
			contentTG.setTransform(vt);
		}
		
		/** scale the y-Value of the content by 1.25 */
		public void scaleUp(){
			Transform3D vt = new Transform3D();
			contentTG.getTransform(vt);
			Matrix4d m = new Matrix4d();
			vt.get(m);
			m.m00 *= 1.25;
			m.m11 *= 1.25;
			m.m22 *= 1.25;
			vt.set(m);
			contentTG.setTransform(vt);
		}
		
		/** scale the y-Value of the content by 0.8 */
		public void scaleDown(){
			Transform3D vt = new Transform3D();
			contentTG.getTransform(vt);
			Matrix4d m = new Matrix4d();
			vt.get(m);
			m.m00 *= 0.8;
			m.m11 *= 0.8;
			m.m22 *= 0.8;
			vt.set(m);
			contentTG.setTransform(vt);
		}
		
		public void keyPressed(KeyEvent e){
			//System.out.println("key pressed!");
			
			if(e.getKeyCode() == KeyEvent.VK_DOWN){
				//System.out.println("down pressed!");
				backward();
			}
			
			if(e.getKeyCode() == KeyEvent.VK_UP){
				//System.out.println("up pressed!");
				forward();
			}
			
			if(e.getKeyCode() == KeyEvent.VK_LEFT){
				//System.out.println("left pressed!");
				rollLeft();
			}
			
			if(e.getKeyCode() == KeyEvent.VK_RIGHT){
				//System.out.println("right pressed!");
				rollRight();
			}
		}
		
		public void keyTyped(KeyEvent e){
		}
		
		public void keyReleased(KeyEvent e){
		}
		
  }
	
  public Canvas3DDisplay() {
		setLayout(new BorderLayout());
  }
	
  /** not needed by this kind of display: empty implementation */
  public void setRange(double[] realRange, double[] imagRange){}
	
	
  /** nothing happens until this method is called  */
  public void setValues(Complex[][] values){
		if(values == null || values.length == 0)
			return;
		this.values = values;
		
		if(plot3d != null)
			remove(plot3d);
		
		plot3d = new Canvas3D(null);
		plot3d.addKeyListener(dirPanel);
		setLayout(new BorderLayout());
		View myView = constructView(plot3d);
		Locale myLocale = constructViewBranch(myView);
		constructContentBranch(myLocale);
		add(plot3d,BorderLayout.CENTER);
		add(dirPanel,BorderLayout.SOUTH);
		validate();
		requestFocus();
  }
	
  /**
	 constructView() takes a Canvas3D reference and constructs
	 a View to display in that Canvas3D.  It uses the default
	 PhysicalBody and PhysicalEnvironment (both required to be
	 set or else the 3D runtime will throw exceptions).  The
	 returned View is used by constructViewBranch() to attach
	 the scene graph's ViewPlatform to a Canvas3D for rendering.
	 
	 */
  private View constructView(Canvas3D plot3d) {
		View myView = new View();
		myView.addCanvas3D(plot3d);
		myView.setPhysicalBody(new PhysicalBody());
		myView.setPhysicalEnvironment(new PhysicalEnvironment());
		return myView;
  }
	
  /**
	 constructViewBranch() takes as input a View which we
	 attached to our Canvas3D in constructView().  It constructs
	 a default view branch for the scene graph, attaches
	 the View to the ViewPlatform, and returns a reference to
	 our Locale for use by constructContentBranch()
	 in creating content for our scene graph.
	 
	 */
  private Locale constructViewBranch(View myView) {
		
		//First, we create the necessary coordinate systems
		//(VirtualUniverse, Locale), container nodes
		//(BranchGroup, TransformGroup), and platform which
		//determines our viewing position and direction (ViewPlatform).
		VirtualUniverse myUniverse = new VirtualUniverse();
		Locale myLocale = new Locale(myUniverse);
		BranchGroup viewBG = new BranchGroup();
		Transform3D vt = new Transform3D();
		vt.setTranslation(new Vector3f(0.0f,0.0f,3.0f));
		Matrix4d m = new Matrix4d();
		m.setIdentity();
		m.rotY(-8.0 * Math.PI/120.0);
		vt.mul(new Transform3D(m));
		viewTG = new TransformGroup(vt);
		viewTG.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		viewTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		ViewPlatform myVP = new ViewPlatform();
		
		//Next, we insert the platform into the transform group,
		//the transform group into the branch group, and the branch
		//group into the locale's branch graph portion of the
		//scene graph.
		viewTG.addChild(myVP);
		viewBG.addChild(viewTG);
		myLocale.addBranchGraph(viewBG);
		
		//Finally, we attach our view to the view platform and we
		//return a reference to our new universe.  We are ready to
		//render 3D content!
		myView.attachViewPlatform(myVP);
		return myLocale;
  }
	
  /**
	 constructContentBranch() is where we specify the 3D graphics
	 content to be rendered into the Locale referenced
	 in the passed parameter.  Nothing is currently specified, so we
	 render an empty universe.
	 
	 */
  private void constructContentBranch(Locale myLocale) {
		BranchGroup contentBG = new BranchGroup();
		//Shape3D testShape = new ColorCube(0.4);
		/* Complex[][] values = {
		 {
		 new Complex(0,0),
		 new Complex(1,0),
		 new Complex(0,0)},
		 {
		 new Complex(0,0),
		 new Complex(1,0),
		 new Complex(0,0)},
		 {
		 new Complex(0,0),
		 new Complex(1,0),
		 new Complex(0,0)}
		 }; */
		
		Shape3D valueShape = ValueShape.getValueShape(values);
		contentTG = new TransformGroup();
		contentTG.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		contentTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		contentTG.addChild(valueShape);
		contentBG.addChild(contentTG);
		//contentBG.compile();
		myLocale.addBranchGraph(contentBG);
  }
	
	
  public static void main(String[] args){
		Canvas3DDisplay ds = new Canvas3DDisplay();
		Frame frame = new Frame();
		frame.addWindowListener(new WindowAdapter() {
					public void windowClosing(WindowEvent e){
						System.exit(0);
					}
				});
		
		frame.add(ds,BorderLayout.CENTER);
		frame.setSize(800,600);
		frame.setVisible(true);
  }
	
}

