package show;

import java.awt.Color;
import java.awt.color.ColorSpace;
import javax.vecmath.*;
import javax.media.j3d.*;
import com.sun.j3d.utils.geometry.*;
import calc.Complex;

/** this object is the (3D) graphical representation of the graph
 of a complex function */


public class ValueShape{
	
  static int debug =1;
	
  public static Shape3D getValueShape(Complex[][] values){
		int n = values.length;
		int m = values[0].length;
		int[] strips = new int[n];
		for(int i=0; i<n;i++)
		  strips[i] = 4*m;
		TriangleStripArray qArr = new TriangleStripArray(n*m*4, TriangleStripArray.COORDINATES | TriangleStripArray.COLOR_3, strips);
		ColoringAttributes ca = new ColoringAttributes();
		ca.setShadeModel(ColoringAttributes.SHADE_GOURAUD);
		Shape3D shape = new Shape3D(qArr);
		Appearance app = new Appearance();
		app.setColoringAttributes(ca);
		shape.setAppearance(app);
		int index = 0;
		Complex lastValidValue = new Complex(0,0);
		for(int i = 0; i < n-1; i++)
			for(int j = 0; j < m-1; j++){
				if(debug > 1)
					System.out.println("Lines : ["+(double) i / n + "," +
															 (values[i][j].abs() - 0.5d) + "," +
															 (double) j / m + "]" +
															 "["+(double) i / n + "," +
															 (values[i][j+1].abs() - 0.5d) + "," +
															 (double) (j+1) / m + "]" +
															 "["+(double) i / n + "," +
															 (values[i][j].abs() - 0.5d) + "," +
															 (double) j / m + "]" +
															 "["+(double) i / n + "," +
															 (values[i][j+1].abs() - 0.5d) + "," +
															 (double) (j+1) / m + "]");
				if(values[i][j].isValid()){
					qArr.setCoordinate(index, new Point3d((double) i / n, values[i][j].abs() - 0.5d, (double) -j / m));
					lastValidValue = values[i][j];
				}
				else
					qArr.setCoordinate(index, new Point3d((double) i / n, lastValidValue.abs() - 0.5d, (double) -j / m));
				qArr.setColor(index++, new Color3f(new Color(Color.HSBtoRGB((float)values[i][j].phi(), 1.0f, 1.0f))));
				
				if(values[i+1][j].isValid()){
					qArr.setCoordinate(index, new Point3d((double) (i+1) / n, values[i+1][j].abs() - 0.5d, (double) - j / m));
					lastValidValue = values[i+1][j];
				}
				else
					qArr.setCoordinate(index, new Point3d((double) i / n, lastValidValue.abs() - 0.5d, (double) -j / m));
				qArr.setColor(index++, new Color3f(new Color(Color.HSBtoRGB((float)values[i+1][j].phi(), 1.0f,1.0f))));
				
				if(values[i][j+1].isValid()){
					qArr.setCoordinate(index, new Point3d((double) i / n, values[i][j+1].abs() - 0.5d, (double) - (j+1) / m));
					lastValidValue = values[i][j+1];
				}
				else
					qArr.setCoordinate(index, new Point3d((double) i / n, lastValidValue.abs() -0.5d, (double) -j / m));
				qArr.setColor(index++, new Color3f(new Color(Color.HSBtoRGB((float)values[i][j+1].phi(), 1.0f,1.0f))));
				
				
				if(values[i+1][j+1].isValid()){
					qArr.setCoordinate(index, new Point3d((double) (i+1) / n, values[i+1][j+1].abs() - 0.5d, (double) - (j+1) / m));
					lastValidValue = values[i+1][j+1];
				}
				else
					qArr.setCoordinate(index, new Point3d((double) i / n, lastValidValue.abs() - 0.5d, (double) -j / m));
				qArr.setColor(index++, new Color3f(new Color(Color.HSBtoRGB((float)values[i+1][j+1].phi(), 1.0f, 1.0f))));
			}
		
		return new Shape3D(qArr);
  }
}
