package show;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import calc.Complex;

/** shows the values of a complex function (handed over in a Complex[][] array)
    as a 2x2 vector field */

public class VectorFieldDisplay extends Field2DDisplay {

  /** initialize canvas as  VectorFieldCanvas in getCanvas(), when called the 
      first time */
  protected Canvas canvas = null;

  public Canvas getCanvas(){
    if(canvas == null)
      canvas = new VectorFieldCanvas();
    return canvas;
  }

  protected Complex[][] values;

  /** this inner class represents the canvas on which everything is
      drawn. We use physical coordinates (depending on the size of the 
      component) and logical coordinates (depending on the given definition 
      range) which are transformed by methods of the outer class */

  public class VectorFieldCanvas extends Canvas {
    public void paint(Graphics g){
      if(n == 0 || m == 0)
	return;

      determineCellSize();
      
      // we use the Java2D-API
      Graphics2D g2 = (Graphics2D) g;
      //g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON); -- too slow!

      // set transform by calling Field2DDisplay.determineTransform()
      determineTransform(getSize().width,
			 getSize().height);

      g2.setTransform(transform);	

      // set line width to one physical pixel (in logical coordinates!)
      g2.setStroke(new BasicStroke((float)(1.0d * (realDif / 
						   ((double)getSize().width))), 
				   BasicStroke.CAP_BUTT, 
				   BasicStroke.JOIN_MITER));
      
      for(int i = 0; i < n; i++)
	for(int j = 0; j < m; j++){
		
	  // paint a vector rooted in the center of the cell and
	  // pointing to (real,imag) with a length scaled to 
	  // x = x * cellWidth /length(x,y)
	  // y = y * cellHeight /length(x,y)

	  double length = values[i][j].abs();

	  if(length != 0.0d && values[i][j].isValid())
	    g2.draw(new Line2D.Double(realRange[0] + (0.5 + (double)i) * cellWidth, 
			imagRange[0] + (0.5 + (double)j) * cellHeight,
			realRange[0] + (0.5 + (double)i + 0.8 * values[i][j].real / length) * cellWidth, 
			imagRange[0] + (0.5 + (double)j + 0.8 * values[i][j].imag / length) * cellHeight));
	  // else System.out.println("values["+i+"]["+j+"] = "+values[i][j]);

		
	}
      // draw coordinates
      g2.setColor(Color.red);

      g2.setStroke(new BasicStroke((float) (2 * imagDif / getSize().height),
				   BasicStroke.CAP_BUTT, 
				   BasicStroke.JOIN_MITER));
      g2.draw(new Line2D.Double(realRange[0], 0, realRange[1], 0));
      g2.setStroke(new BasicStroke((float) (2 * realDif / getSize().width),
				   BasicStroke.CAP_BUTT, 
				   BasicStroke.JOIN_MITER));
      g2.draw(new Line2D.Double(0, imagRange[0], 0, imagRange[1]));

    }
  }

  public VectorFieldDisplay(){

    // initialize, so that paint() doesn't complain
    Complex[][] nullValues = {{}};
    setValues(nullValues);
  }
     
  public void setValues(Complex[][] inValues){
    n = inValues.length;
    m = inValues[0].length;

    values = new Complex[n][m];
    for(int i = 0; i < n; i++)
      for(int j = 0; j < m; j++)
	values[i][j] = new Complex(inValues[i][j]);
  }    



}
	
